@@ -20,7 +20,6 @@
 
 
 using namespace OIIO;
-using OIIO::Strutil::print;
 
 static std::string uninitialized  = "uninitialized \001 HHRU dfvAS: efjl";
 static std::string dataformatname = "";
@@ -103,7 +102,7 @@ getargs(int argc, char* argv[])
                 nullptr);
     // clang-format on
     if (ap.parse(argc, (const char**)argv) < 0) {
-        print(stderr, "{}\n", ap.geterror());
+        OIIO::print(stderr, "{}\n", ap.geterror());
         ap.usage();
         ap.abort();
         return_code = EXIT_FAILURE;
@@ -117,7 +116,7 @@ getargs(int argc, char* argv[])
     }
 
     if (filenames.size() != 2 && !inplace) {
-        print(
+        OIIO::print(
             stderr,
             "iconvert: Must have both an input and output filename specified.\n");
         ap.usage();
@@ -126,14 +125,14 @@ getargs(int argc, char* argv[])
         return;
     }
     if (filenames.size() == 0 && inplace) {
-        print(stderr, "iconvert: Must have at least one filename\n");
+        OIIO::print(stderr, "iconvert: Must have at least one filename\n");
         ap.usage();
         ap.abort();
         return_code = EXIT_FAILURE;
         return;
     }
     if (((int)rotcw + (int)rotccw + (int)rot180 + (orientation > 0)) > 1) {
-        print(
+        OIIO::print(
             stderr,
             "iconvert: more than one of --rotcw, --rotccw, --rot180, --orientation\n");
         ap.usage();
@@ -151,9 +150,9 @@ DateTime_to_time_t(string_view datetime, time_t& timet)
     int year, month, day, hour, min, sec;
     if (!Strutil::scan_datetime(datetime, year, month, day, hour, min, sec))
         return false;
-    // print("{}:{}:{} {}:{}:{}\n", year, month, day, hour, min, sec);
+    // OIIO::print("{}:{}:{} {}:{}:{}\n", year, month, day, hour, min, sec);
     struct tm tmtime;
-    time_t now;
+    time_t now = time(nullptr);
     Sysutil::get_local_time(&now, &tmtime);  // fill in defaults
     tmtime.tm_sec  = sec;
     tmtime.tm_min  = min;
@@ -314,13 +313,14 @@ static bool
 convert_file(const std::string& in_filename, const std::string& out_filename)
 {
     if (noclobber && Filesystem::exists(out_filename)) {
-        print(stderr, "iconvert ERROR: Output file already exists \"{}\"\n",
-              out_filename);
+        OIIO::print(stderr,
+                    "iconvert ERROR: Output file already exists \"{}\"\n",
+                    out_filename);
         return false;
     }
 
     if (verbose) {
-        print("Converting {} to {}\n", in_filename, out_filename);
+        OIIO::print("Converting {} to {}\n", in_filename, out_filename);
         fflush(stdout);
     }
 
@@ -333,10 +333,11 @@ convert_file(const std::string& in_filename, const std::string& out_filename)
     auto in = ImageInput::open(in_filename);
     if (!in) {
         std::string err = geterror();
-        print(stderr, "iconvert ERROR: {}\n",
-              (err.length() ? err
-                            : Strutil::fmt::format("Could not open \"{}\"",
-                                                   in_filename)));
+        OIIO::print(stderr, "iconvert ERROR: {}\n",
+                    (err.length()
+                         ? err
+                         : Strutil::fmt::format("Could not open \"{}\"",
+                                                in_filename)));
         return false;
     }
     ImageSpec inspec         = in->spec();
@@ -345,7 +346,7 @@ convert_file(const std::string& in_filename, const std::string& out_filename)
     // Find an ImageIO plugin that can open the output file, and open it
     auto out = ImageOutput::create(tempname);
     if (!out) {
-        print(
+        OIIO::print(
             stderr,
             "iconvert ERROR: Could not find an ImageIO plugin to write \"{}\": {}\n",
             out_filename, geterror());
@@ -375,10 +376,11 @@ convert_file(const std::string& in_filename, const std::string& out_filename)
     bool mip_to_subimage_warning = false;
     for (int subimage = 0; ok && in->seek_subimage(subimage, 0); ++subimage) {
         if (subimage > 0 && !out->supports("multiimage")) {
-            print(stderr,
-                  "iconvert WARNING: {} does not support multiple subimages.\n"
-                  "\tOnly the first subimage has been copied.\n",
-                  out->format_name());
+            OIIO::print(
+                stderr,
+                "iconvert WARNING: {} does not support multiple subimages.\n"
+                "\tOnly the first subimage has been copied.\n",
+                out->format_name());
             break;  // we're done
         }
 
@@ -398,17 +400,18 @@ convert_file(const std::string& in_filename, const std::string& out_filename)
                     mode = ImageOutput::AppendSubimage;  // use if we must
                     if (!mip_to_subimage_warning
                         && strcmp(out->format_name(), "tiff")) {
-                        print(stderr,
-                              "iconvert WARNING: {} does not support MIPmaps.\n"
-                              "\tStoring the MIPmap levels in subimages.\n",
-                              out->format_name());
+                        OIIO::print(
+                            stderr,
+                            "iconvert WARNING: {} does not support MIPmaps.\n"
+                            "\tStoring the MIPmap levels in subimages.\n",
+                            out->format_name());
                     }
                     mip_to_subimage_warning = true;
                 } else {
-                    print(stderr,
-                          "iconvert WARNING: {} does not support MIPmaps.\n"
-                          "\tOnly the first level has been copied.\n",
-                          out->format_name());
+                    OIIO::print(stderr,
+                                "iconvert WARNING: {} does not support MIPmaps.\n"
+                                "\tOnly the first level has been copied.\n",
+                                out->format_name());
                     break;  // on to the next subimage
                 }
                 ok = out->open(tempname.c_str(), outspec, mode);
@@ -427,11 +430,11 @@ convert_file(const std::string& in_filename, const std::string& out_filename)
             }
             if (!ok) {
                 std::string err = out->geterror();
-                print(stderr, "iconvert ERROR: {}\n",
-                      (err.length()
-                           ? err
-                           : Strutil::fmt::format("Could not open \"{}\"",
-                                                  out_filename)));
+                OIIO::print(stderr, "iconvert ERROR: {}\n",
+                            (err.length()
+                                 ? err
+                                 : Strutil::fmt::format("Could not open \"{}\"",
+                                                        out_filename)));
                 ok = false;
                 break;
             }
@@ -450,9 +453,10 @@ convert_file(const std::string& in_filename, const std::string& out_filename)
             if (!nocopy) {
                 ok = out->copy_image(in.get());
                 if (!ok)
-                    print(stderr,
-                          "iconvert ERROR copying \"{}\" to \"{}\" :\n\t{}\n",
-                          in_filename, out_filename, out->geterror());
+                    OIIO::print(
+                        stderr,
+                        "iconvert ERROR copying \"{}\" to \"{}\" :\n\t{}\n",
+                        in_filename, out_filename, out->geterror());
             } else {
                 // Need to do it by hand for some reason.  Future expansion in which
                 // only a subset of channels are copied, or some such.
@@ -460,13 +464,14 @@ convert_file(const std::string& in_filename, const std::string& out_filename)
                 ok = in->read_image(subimage, miplevel, 0, outspec.nchannels,
                                     outspec.format, &pixels[0]);
                 if (!ok) {
-                    print(stderr, "iconvert ERROR reading \"{}\": {}\n",
-                          in_filename, in->geterror());
+                    OIIO::print(stderr, "iconvert ERROR reading \"{}\": {}\n",
+                                in_filename, in->geterror());
                 } else {
                     ok = out->write_image(outspec.format, &pixels[0]);
                     if (!ok)
-                        print(stderr, "iconvert ERROR writing \"{}\": {}\n",
-                              out_filename, out->geterror());
+                        OIIO::print(stderr,
+                                    "iconvert ERROR writing \"{}\": {}\n",
+                                    out_filename, out->geterror());
                 }
             }
 
