@@ -69,7 +69,8 @@ setup_filter(const ImageSpec& dstspec, const ImageSpec& srcspec,
 
 
 static TypeDesc
-set_prman_options(TypeDesc out_dataformat, ImageSpec& configspec)
+set_prman_options(TypeDesc out_dataformat, ImageSpec& configspec,
+                  bool is_bumpslopes)
 {
     // Force separate planar image handling, and also emit prman metadata
     configspec.attribute("planarconfig", "separate");
@@ -98,6 +99,8 @@ set_prman_options(TypeDesc out_dataformat, ImageSpec& configspec)
     // specifying -half or -float make 32x32 tile size
     if (out_dataformat == TypeDesc::DOUBLE)
         out_dataformat = TypeDesc::FLOAT;
+    if (is_bumpslopes && !out_dataformat.is_floating_point())
+        out_dataformat = TypeDesc::FLOAT;
     if (out_dataformat == TypeDesc::HALF || out_dataformat == TypeDesc::FLOAT) {
         configspec.tile_width  = 32;
         configspec.tile_height = 32;
@@ -128,26 +131,29 @@ datestring(time_t t)
 {
     struct tm mytm;
     Sysutil::get_local_time(&t, &mytm);
-    return Strutil::sprintf("%4d:%02d:%02d %02d:%02d:%02d", mytm.tm_year + 1900,
-                            mytm.tm_mon + 1, mytm.tm_mday, mytm.tm_hour,
-                            mytm.tm_min, mytm.tm_sec);
+    return Strutil::fmt::format("{:4d}:{:02d}:{:02d} {:02d}:{:02d}:{:02d}",
+                                mytm.tm_year + 1900, mytm.tm_mon + 1,
+                                mytm.tm_mday, mytm.tm_hour, mytm.tm_min,
+                                mytm.tm_sec);
 }
 
 
 
 template<class SRCTYPE>
 static void
-interppixel_NDC_clamped(const ImageBuf& buf, float x, float y, float* pixel,
-                        bool envlatlmode)
+interppixel_NDC(const ImageBuf& buf, float x, float y, span<float> pixel,
+                bool envlatlmode, ImageBuf::ConstIterator<SRCTYPE>& it,
+                ImageBuf::WrapMode wrapmode)
 {
-    int fx = buf.spec().full_x;
-    int fy = buf.spec().full_y;
-    int fw = buf.spec().full_width;
-    int fh = buf.spec().full_height;
+    const ImageSpec& spec(buf.spec());
+    int fx = spec.full_x;
+    int fy = spec.full_y;
+    int fw = spec.full_width;
+    int fh = spec.full_height;
     x      = static_cast<float>(fx) + x * static_cast<float>(fw);
     y      = static_cast<float>(fy) + y * static_cast<float>(fh);
 
-    int n     = buf.spec().nchannels;
+    int n     = spec.nchannels;
     float *p0 = OIIO_ALLOCA(float, 4 * n), *p1 = p0 + n, *p2 = p1 + n,
           *p3 = p2 + n;
 
@@ -159,8 +165,7 @@ interppixel_NDC_clamped(const ImageBuf& buf, float x, float y, float* pixel,
     yfrac = floorfrac(y, &ytexel);
 
     // Get the four texels
-    ImageBuf::ConstIterator<SRCTYPE> it(
-        buf, ROI(xtexel, xtexel + 2, ytexel, ytexel + 2), ImageBuf::WrapClamp);
+    it.rerange(xtexel, xtexel + 2, ytexel, ytexel + 2, 0, 1, wrapmode);
     for (int c = 0; c < n; ++c)
         p0[c] = it[c];
     ++it;
@@ -180,8 +185,10 @@ interppixel_NDC_clamped(const ImageBuf& buf, float x, float y, float* pixel,
         // wrong will tend to over-represent the high latitudes in
         // low-res MIP levels.  We fold the area weighting into our
         // linear interpolation by adjusting yfrac.
-        int ynext = OIIO::clamp(ytexel + 1, buf.ymin(), buf.ymax());
-        ytexel    = OIIO::clamp(ytexel, buf.ymin(), buf.ymax());
+        int ymin  = spec.y;
+        int ymax  = ymin + spec.height - 1;
+        int ynext = OIIO::clamp(ytexel + 1, ymin, ymax);
+        ytexel    = OIIO::clamp(ytexel, ymin, ymax);
         float w0  = (1.0f - yfrac)
                    * sinf((float)M_PI * (ytexel + 0.5f) / (float)fh);
         float w1 = yfrac * sinf((float)M_PI * (ynext + 0.5f) / (float)fh);
@@ -189,7 +196,7 @@ interppixel_NDC_clamped(const ImageBuf& buf, float x, float y, float* pixel,
     }
 
     // Bilinearly interpolate
-    bilerp(p0, p1, p2, p3, xfrac, yfrac, n, pixel);
+    bilerp(p0, p1, p2, p3, xfrac, yfrac, n, pixel.data());
 }
 
 
@@ -210,22 +217,23 @@ resize_block_(ImageBuf& dst, const ImageBuf& src, ROI roi, bool envlatlmode)
            || srcspec.z + srcspec.depth < srcspec.full_z + srcspec.full_depth);
 
     const ImageSpec& dstspec(dst.spec());
-    float* pel    = OIIO_ALLOCA(float, dstspec.nchannels);
-    float xoffset = (float)dstspec.full_x;
-    float yoffset = (float)dstspec.full_y;
-    float xscale  = 1.0f / (float)dstspec.full_width;
-    float yscale  = 1.0f / (float)dstspec.full_height;
-    int nchannels = dst.nchannels();
+    span<float> pel = OIIO_ALLOCA_SPAN(float, dstspec.nchannels);
+    float xoffset   = (float)dstspec.full_x;
+    float yoffset   = (float)dstspec.full_y;
+    float xscale    = 1.0f / (float)dstspec.full_width;
+    float yscale    = 1.0f / (float)dstspec.full_height;
+    int nchannels   = dst.nchannels();
     OIIO_DASSERT(dst.spec().format == TypeFloat);
+    ImageBuf::WrapMode wrapmode = src_is_crop ? ImageBuf::WrapBlack
+                                              : ImageBuf::WrapClamp;
+    ImageBuf::ConstIterator<SRCTYPE> srcit(src);
     ImageBuf::Iterator<float> d(dst, roi);
     for (int y = y0; y < y1; ++y) {
         float t = (y + 0.5f) * yscale + yoffset;
         for (int x = x0; x < x1; ++x, ++d) {
             float s = (x + 0.5f) * xscale + xoffset;
-            if (src_is_crop)
-                src.interppixel_NDC(s, t, pel);
-            else
-                interppixel_NDC_clamped<SRCTYPE>(src, s, t, pel, envlatlmode);
+            interppixel_NDC<SRCTYPE>(src, s, t, pel, envlatlmode, srcit,
+                                     wrapmode);
             for (int c = 0; c < nchannels; ++c)
                 d[c] = pel[c];
         }
@@ -310,7 +318,7 @@ resize_block(ImageBuf& dst, const ImageBuf& src, ROI roi, bool envlatlmode,
         !envlatlmode &&                          // not latlong wrap mode
         roi.xbegin == 0 &&                       // Region x at origin
         dstspec.width == roi.width() &&          // Full width ROI
-        dstspec.width == (srcspec.width / 2) &&  // Src is 2x resize
+        (2 * dstspec.width) == srcspec.width &&  // Src is 2x resize
         dstspec.format == srcspec.format &&      // Same formats
         dstspec.x == 0 && dstspec.y == 0 &&      // Not a crop or overscan
         srcspec.x == 0 && srcspec.y == 0) {
@@ -334,7 +342,7 @@ check_nan_block(const ImageBuf& src, ROI roi, int& found_nonfinite)
 {
     int x0 = roi.xbegin, x1 = roi.xend, y0 = roi.ybegin, y1 = roi.yend;
     const ImageSpec& spec(src.spec());
-    float* pel = OIIO_ALLOCA(float, spec.nchannels);
+    span<float> pel = OIIO_ALLOCA_SPAN(float, spec.nchannels);
     for (int y = y0; y < y1; ++y) {
         for (int x = x0; x < x1; ++x) {
             src.getpixel(x, y, pel);
@@ -384,8 +392,9 @@ lightprobe_to_envlatl(ImageBuf& dst, const ImageBuf& src, bool y_is_up,
         const ImageSpec& dstspec(dst.spec());
         int nchannels = dstspec.nchannels;
 
-        float* pixel = OIIO_ALLOCA(float, nchannels);
+        span<float> pixel = OIIO_ALLOCA_SPAN(float, nchannels);
         float dw = dstspec.width, dh = dstspec.height;
+        ImageBuf::ConstIterator<SRCTYPE> srcit(src);
         for (ImageBuf::Iterator<float> d(dst, roi); !d.done(); ++d) {
             Imath::V3f V = latlong_to_dir((d.x() + 0.5f) / dw,
                                           (dh - 1.0f - d.y() + 0.5f) / dh,
@@ -393,8 +402,8 @@ lightprobe_to_envlatl(ImageBuf& dst, const ImageBuf& src, bool y_is_up,
             float r      = M_1_PI * acosf(V[2]) / hypotf(V[0], V[1]);
             float u      = (V[0] * r + 1.0f) * 0.5f;
             float v      = (V[1] * r + 1.0f) * 0.5f;
-            interppixel_NDC_clamped<SRCTYPE>(src, float(u), float(v), pixel,
-                                             false);
+            interppixel_NDC<SRCTYPE>(src, float(u), float(v), pixel, false,
+                                     srcit, ImageBuf::WrapClamp);
             for (int c = roi.chbegin; c < roi.chend; ++c)
                 d[c] = pixel[c];
         }
@@ -405,46 +414,67 @@ lightprobe_to_envlatl(ImageBuf& dst, const ImageBuf& src, bool y_is_up,
 
 
 
+struct SlopeFilter {
+    float ds[9];
+    float dt[9];
+    float total;
+};
+
+static constexpr SlopeFilter sobel {
+    { -1.0f, 0.0f, 1.0f, -2.0f, 0.0f, 2.0f, -1.0f, 0.0f, 1.0f },
+    { -1.0f, -2.0f, -1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 2.0f, 1.0f },
+    8.0f
+};
+
+static constexpr SlopeFilter centraldiff {
+    { 0.0f, 0.0f, 0.0f, -1.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f },
+    { 0.0f, -1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f },
+    2.0f
+};
+
 // compute slopes in pixel space using a Sobel gradient filter
-template<class SRCTYPE>
+template<class SRCTYPE, SlopeFilter const& filter>
 static void
-sobel_gradient(const ImageBuf& src, const ImageBuf::Iterator<float>& dstpix,
+slope_gradient(const ImageBuf& src, const ImageBuf::Iterator<float>& dstpix,
                float* h, float* dh_ds, float* dh_dt)
 {
-    static const float sobelweight_ds[9] = { -1.0f, 0.0f,  1.0f, -2.0f, 0.0f,
-                                             2.0f,  -1.0f, 0.0f, 1.0f };
-    static const float sobelweight_dt[9] = { -1.0f, -2.0f, -1.0f, 0.0f, 0.0f,
-                                             0.0f,  1.0f,  2.0f,  1.0f };
-
     *dh_ds = *dh_dt = 0.0f;
 
     ImageBuf::ConstIterator<SRCTYPE> srcpix(src, dstpix.x() - 1, dstpix.x() + 2,
                                             dstpix.y() - 1, dstpix.y() + 2, 0,
                                             1, ImageBuf::WrapClamp);
     for (int i = 0; !srcpix.done(); ++srcpix, ++i) {
-        // accumulate to dh_ds and dh_dt using corresponding sobel 3x3 weights
+        // accumulate to dh_ds and dh_dt using corresponding 3x3 gradient weights
         float srcval = srcpix[0];
-        *dh_ds += sobelweight_ds[i] * srcval;
-        *dh_dt += sobelweight_dt[i] * srcval;
+        *dh_ds += filter.ds[i] * srcval;
+        *dh_dt += filter.dt[i] * srcval;
         if (i == 4)
             *h = srcval;
     }
 
-    *dh_ds = *dh_ds / 8.0f;  // sobel normalization
-    *dh_dt = *dh_dt / 8.0f;
+    *dh_ds = *dh_ds / filter.total;  // normalization
+    *dh_dt = *dh_dt / filter.total;
 }
 
 
 
 // compute slopes from normal in s,t space
 // Note: because we use getpixel(), it works for all src pixel types.
+template<bool is_centered>
 static void
 normal_gradient(const ImageBuf& src, const ImageBuf::Iterator<float>& dstpix,
                 float* h, float* dh_ds, float* dh_dt)
 {
     // assume a normal defined in the tangent space
     float n[3];
-    src.getpixel(dstpix.x(), dstpix.y(), n, 3);
+    src.getpixel(dstpix.x(), dstpix.y(), make_span(n));
+
+    if (!is_centered) {  // pixel data is on [0,1] instead of [-1,1]
+        n[0] -= 0.5f;
+        n[1] -= 0.5f;
+        n[2] -= 0.5f;
+    }
+
     *h     = -1.0f;
     *dh_ds = -n[0] / n[2];
     *dh_dt = -n[1] / n[2];
@@ -455,8 +485,10 @@ normal_gradient(const ImageBuf& src, const ImageBuf::Iterator<float>& dstpix,
 template<class SRCTYPE>
 static bool
 bump_to_bumpslopes(ImageBuf& dst, const ImageBuf& src,
-                   const ImageSpec& configspec, std::ostream& outstream,
-                   ROI roi = ROI::All(), int nthreads = 0)
+                   const ImageSpec& configspec,
+                   const ImageBufAlgo::PixelStats& pixel_stats,
+                   std::ostream& outstream, ROI roi = ROI::All(),
+                   int nthreads = 0)
 {
     if (!dst.initialized() || dst.nchannels() != 6
         || dst.spec().format != TypeDesc::FLOAT)
@@ -466,42 +498,80 @@ bump_to_bumpslopes(ImageBuf& dst, const ImageBuf& src,
     void (*bump_filter)(const ImageBuf&, const ImageBuf::Iterator<float>&,
                         float*, float*, float*);
 
-    bump_filter = &sobel_gradient<SRCTYPE>;
-
-    float res_x = 1.0f;
-    float res_y = 1.0f;
-
     string_view bumpformat = configspec.get_string_attribute(
         "maketx:bumpformat");
 
+    bool is_height;  // true => height, false => normal
     if (Strutil::iequals(bumpformat, "height"))
-        bump_filter = &sobel_gradient<
-            SRCTYPE>;  // default one considering height value in channel 0
+        is_height = true;
     else if (Strutil::iequals(bumpformat, "normal")) {
         if (src.spec().nchannels < 3) {
             outstream
                 << "maketx ERROR: normal map requires 3 channels input map.\n";
             return false;
         }
-        bump_filter = &normal_gradient;
+        is_height = false;
     } else if (Strutil::iequals(
                    bumpformat,
                    "auto")) {  // guess input bump format by analyzing channel count and component
         if (src.spec().nchannels > 2
             && !ImageBufAlgo::isMonochrome(src))  // maybe it's a normal map?
-            bump_filter = &normal_gradient;
+            is_height = false;
+        else
+            is_height = true;
     } else {
         outstream << "maketx ERROR: Unknown input bump format " << bumpformat
                   << ". Valid formats are height, normal or auto\n";
         return false;
     }
 
-    float uv_scale = configspec.get_float_attribute(
+    string_view slopefilter = configspec.get_string_attribute(
+        "maketx:slopefilter");
+    string_view bumprange = configspec.get_string_attribute("maketx:bumprange");
+
+    if (is_height) {
+        if (Strutil::iequals(slopefilter, "sobel"))  // default
+            bump_filter = &slope_gradient<SRCTYPE, sobel>;
+        else if (Strutil::iequals(slopefilter, "centraldiff"))
+            bump_filter = &slope_gradient<SRCTYPE, centraldiff>;
+        else {
+            outstream << "maketx ERROR: Unknown input slope kernel "
+                      << slopefilter
+                      << ". Valid kernels are sobel, centraldiff\n";
+            return false;
+        }
+    } else {  // is normal map
+        if (Strutil::iequals(bumprange, "centered"))
+            bump_filter = &normal_gradient<true>;
+        else if (Strutil::iequals(bumprange, "positive"))
+            bump_filter = &normal_gradient<false>;
+        else if (Strutil::iequals(bumprange, "auto")) {
+            if (!src.spec().format.is_signed())
+                bump_filter = &normal_gradient<false>;
+            else {  // check if there are negative values in the image
+                float min_val = *std::min_element(std::begin(pixel_stats.min),
+                                                  std::end(pixel_stats.min));
+                if (min_val < 0.0f)
+                    bump_filter = &normal_gradient<true>;
+                else
+                    bump_filter = &normal_gradient<false>;
+            }
+        } else {
+            outstream << "maketx ERROR: Unknown input bump range " << bumprange
+                      << ". Valid ranges are centered, positive, auto\n";
+            return false;
+        }
+    }
+
+    float res_x = 1.0f;
+    float res_y = 1.0f;
+
+    const float uv_scale = configspec.get_float_attribute(
         "maketx:uvslopes_scale",
         configspec.get_float_attribute("uvslopes_scale"));
 
     // If the input is an height map, does the derivatives needs to be UV normalized and scaled?
-    if (bump_filter == &sobel_gradient<SRCTYPE> && uv_scale != 0) {
+    if (is_height && uv_scale != 0) {
         if (uv_scale < 0) {
             outstream
                 << "maketx ERROR: Invalid uvslopes_scale value. The value must be >=0.\n";
@@ -513,6 +583,13 @@ bump_to_bumpslopes(ImageBuf& dst, const ImageBuf& src,
         res_y = (float)src.spec().height / uv_scale;
     }
 
+    const float scale  = configspec.get_float_attribute("maketx:bumpscale",
+                                                        1.0f);
+    const bool inverts = configspec.get_int_attribute("maketx:bumpinverts")
+                         != 0;
+    const bool invertt = configspec.get_int_attribute("maketx:bumpinvertt")
+                         != 0;
+
     ImageBufAlgo::parallel_image(roi, nthreads, [&](ROI roi) {
         // iterate on destination image
         for (ImageBuf::Iterator<float> d(dst, roi); !d.done(); ++d) {
@@ -520,6 +597,11 @@ bump_to_bumpslopes(ImageBuf& dst, const ImageBuf& src,
             float dhds;
             float dhdt;
             bump_filter(src, d, &h, &dhds, &dhdt);
+
+            // scale/invert
+            dhds *= scale * (inverts ? -1.0f : 1.0f);
+            dhdt *= scale * (invertt ? -1.0f : 1.0f);
+
             // h = height or h = -1.0f if a normal map
             d[0] = h;
             // first moments
@@ -539,9 +621,9 @@ bump_to_bumpslopes(ImageBuf& dst, const ImageBuf& src,
 static void
 fix_latl_edges(ImageBuf& buf)
 {
-    int n        = buf.nchannels();
-    float* left  = OIIO_ALLOCA(float, n);
-    float* right = OIIO_ALLOCA(float, n);
+    int n             = buf.nchannels();
+    span<float> left  = OIIO_ALLOCA_SPAN(float, n);
+    span<float> right = OIIO_ALLOCA_SPAN(float, n);
 
     // Make the whole first and last row be solid, since they are exactly
     // on the pole
@@ -614,7 +696,7 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
              std::ostream& outstream, double& stat_writetime,
              double& stat_miptime, size_t& peak_mem)
 {
-    using OIIO::pvt::errorfmt;
+    using OIIO::errorfmt;
     using OIIO::Strutil::sync::print;  // Be sure to use synchronized one
     bool envlatlmode       = (mode == ImageBufAlgo::MakeTxEnvLatl);
     bool orig_was_overscan = (img->spec().x || img->spec().y || img->spec().z
@@ -694,10 +776,10 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
 
     // Write out the image
     if (verbose) {
-        print(outstream, "  Writing file: {}\n", outputfilename);
-        print(outstream, "  Filter \"{}\"\n", filtername);
-        print(outstream, "  Top level is {}x{}\n", outspec.width,
-              outspec.height);
+        OIIO::print(outstream, "  Writing file: {}\n", outputfilename);
+        OIIO::print(outstream, "  Filter \"{}\"\n", filtername);
+        OIIO::print(outstream, "  Top level is {}x{}\n", outspec.width,
+                    outspec.height);
     }
 
     if (clamp_half) {
@@ -718,13 +800,14 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
     if (verbose) {
         size_t mem = Sysutil::memory_used(true);
         peak_mem   = std::max(peak_mem, mem);
-        print(outstream, "    {:-15s} ({})  write {}\n", formatres(outspec),
-              Strutil::memformat(mem), Strutil::timeintervalformat(wtime, 2));
+        OIIO::print(outstream, "    {:15s} ({})  write {}\n",
+                    formatres(outspec), Strutil::memformat(mem),
+                    Strutil::timeintervalformat(wtime, 2));
     }
 
     if (mipmap) {  // Mipmap levels:
         if (verbose)
-            print(outstream, "  Mipmapping...\n");
+            OIIO::print(outstream, "  Mipmapping...\n");
         std::vector<std::string> mipimages;
         std::string mipimages_unsplit = configspec.get_string_attribute(
             "maketx:mipimages");
@@ -747,8 +830,9 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
                     outstream << "WARNING: Custom mip level \"" << mipimages[0]
                               << " had the wrong number of channels.\n";
                     std::shared_ptr<ImageBuf> t(new ImageBuf(smallspec));
-                    ImageBufAlgo::channels(*t, *small, outspec.nchannels, NULL,
-                                           NULL, NULL, true);
+                    ImageBufAlgo::channels(*t, *small, outspec.nchannels,
+                                           cspan<int>(), cspan<float>(),
+                                           cspan<std::string>(), true);
                     std::swap(t, small);
                 }
                 smallspec.tile_width  = outspec.tile_width;
@@ -757,7 +841,9 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
                 mipimages.erase(mipimages.begin());
             } else {
                 // Resize a factor of two smaller
-                smallspec        = outspec;
+                smallspec = outspec;
+                if (!configspec.get_int_attribute("maketx:mipmap_metadata"))
+                    smallspec.extra_attribs.free();
                 smallspec.width  = img->spec().width;
                 smallspec.height = img->spec().height;
                 smallspec.depth  = img->spec().depth;
@@ -804,16 +890,16 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
                         return false;
                     }
                     if (verbose) {
-                        print(outstream,
-                              "  Downsampling filter \"{}\" width = {}",
-                              filter->name(), filter->width());
+                        OIIO::print(outstream,
+                                    "  Downsampling filter \"{}\" width = {}",
+                                    filter->name(), filter->width());
                         if (sharpen > 0.0f)
-                            print(
+                            OIIO::print(
                                 outstream,
                                 ", sharpening {} with {} unsharp mask {} the resize",
                                 sharpen, sharpenfilt,
                                 (sharpen_first ? "before" : "after"));
-                        print(outstream, "\n");
+                        OIIO::print(outstream, "\n");
                     }
                     if (do_highlight_compensation)
                         ImageBufAlgo::rangecompress(*img, *img);
@@ -826,7 +912,8 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
                             errorfmt("{}", sharp->geterror());
                         std::swap(img, sharp);
                     }
-                    ImageBufAlgo::resize(*small, *img, filter);
+                    ImageBufAlgo::resize(*small, *img,
+                                         { make_pv("filterptr", filter) });
                     if (sharpen > 0.0f && !sharpen_first) {
                         std::shared_ptr<ImageBuf> sharp(new ImageBuf);
                         bool uok = ImageBufAlgo::unsharp_mask(*sharp, *small,
@@ -879,18 +966,18 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
             if (verbose) {
                 size_t mem = Sysutil::memory_used(true);
                 peak_mem   = std::max(peak_mem, mem);
-                print(outstream, "    {:-15s} ({})  downres {} write {}\n",
-                      formatres(smallspec), Strutil::memformat(mem),
-                      Strutil::timeintervalformat(this_miptime, 2),
-                      Strutil::timeintervalformat(wtime, 2));
+                OIIO::print(outstream, "    {:15s} ({})  downres {} write {}\n",
+                            formatres(smallspec), Strutil::memformat(mem),
+                            Strutil::timeintervalformat(this_miptime, 2),
+                            Strutil::timeintervalformat(wtime, 2));
             }
             std::swap(img, small);
         }
     }
 
     if (verbose)
-        print(outstream, "  Wrote file: {}  ({})\n", outputfilename,
-              Strutil::memformat(Sysutil::memory_used(true)));
+        OIIO::print(outstream, "  Wrote file: {}  ({})\n", outputfilename,
+                    Strutil::memformat(Sysutil::memory_used(true)));
     writetimer.reset();
     writetimer.start();
     if (!out->close()) {
@@ -976,7 +1063,7 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
                   std::string filename, std::string outputfilename,
                   const ImageSpec& _configspec, std::ostream* outstream_ptr)
 {
-    using OIIO::pvt::errorfmt;
+    using OIIO::errorfmt;
     using OIIO::Strutil::sync::print;  // Be sure to use synchronized one
     OIIO_ASSERT(mode >= 0 && mode < ImageBufAlgo::_MakeTxLast);
     double stat_readtime         = 0;
@@ -987,14 +1074,14 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
     size_t peak_mem              = 0;
     Timer alltime;
 
-#define STATUS(task, timer)                                 \
-    {                                                       \
-        size_t mem = Sysutil::memory_used(true);            \
-        peak_mem   = std::max(peak_mem, mem);               \
-        if (verbose)                                        \
-            print(outstream, "  {:-25s} {}   ({})\n", task, \
-                  Strutil::timeintervalformat(timer, 2),    \
-                  Strutil::memformat(mem));                 \
+#define STATUS(task, timer)                                      \
+    {                                                            \
+        size_t mem = Sysutil::memory_used(true);                 \
+        peak_mem   = std::max(peak_mem, mem);                    \
+        if (verbose)                                             \
+            OIIO::print(outstream, "  {:25s} {}   ({})\n", task, \
+                        Strutil::timeintervalformat(timer, 2),   \
+                        Strutil::memformat(mem));                \
     }
 
     ImageSpec configspec = _configspec;
@@ -1033,8 +1120,9 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         src.reset(new ImageBuf(*input));
     } else {
         // Image buffer supplied that has pixels -- wrap it
-        src.reset(new ImageBuf(input->name(), input->spec(),
-                               (void*)input->localpixels()));
+        src.reset(new ImageBuf(input->spec(),
+                               make_cspan((std::byte*)input->localpixels(),
+                                          input->spec().image_bytes())));
     }
     OIIO_DASSERT(src.get());
 
@@ -1124,11 +1212,14 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
     if (configspec.format != TypeDesc::UNKNOWN)
         out_dataformat = configspec.format;
 
+    const bool is_bumpslopes = (mode == ImageBufAlgo::MakeTxBumpWithSlopes);
+
     // We cannot compute the prman / oiio options until after out_dataformat
     // has been determined, as it's required (and can potentially change
     // out_dataformat too!)
     if (configspec.get_int_attribute("maketx:prman_options"))
-        out_dataformat = set_prman_options(out_dataformat, configspec);
+        out_dataformat = set_prman_options(out_dataformat, configspec,
+                                           is_bumpslopes);
     else if (configspec.get_int_attribute("maketx:oiio_options"))
         out_dataformat = set_oiio_options(out_dataformat, configspec);
 
@@ -1154,6 +1245,7 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
     stat_readtime += alltime.lap();
     STATUS(Strutil::fmt::format("read \"{}\"", src->name()), stat_readtime);
 
+
     if (mode == ImageBufAlgo::MakeTxEnvLatlFromLightProbe) {
         ImageSpec newspec = src->spec();
         newspec.width = newspec.full_width = src->spec().width;
@@ -1173,8 +1265,13 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         src  = latlong;
     }
 
-    const bool is_bumpslopes = (mode == ImageBufAlgo::MakeTxBumpWithSlopes);
     if (is_bumpslopes) {
+        ImageBufAlgo::PixelStats src_pixel_stats;
+        if (Strutil::iequals(configspec.get_string_attribute("maketx:bumprange",
+                                                             "auto"),
+                             "auto"))
+            src_pixel_stats = ImageBufAlgo::computePixelStats(*src);
+
         ImageSpec newspec  = src->spec();
         newspec.tile_width = newspec.tile_height = 0;
         newspec.set_format(TypeDesc::FLOAT);
@@ -1190,7 +1287,7 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         bool ok;
         OIIO_DISPATCH_COMMON_TYPES(ok, "bump_to_bumpslopes", bump_to_bumpslopes,
                                    src->spec().format, *bumpslopes, *src,
-                                   configspec, outstream);
+                                   configspec, src_pixel_stats, outstream);
         // bump_to_bumpslopes(*bumpslopes, *src);
         mode = ImageBufAlgo::MakeTxTexture;
         src  = bumpslopes;
@@ -1323,9 +1420,8 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
             newspec.full_width  = newspec.width;
             newspec.full_height = newspec.height;
             newspec.full_depth  = newspec.depth;
-            std::string name    = std::string(src->name()) + ".constant_color";
-            src->reset(name, newspec);
-            ImageBufAlgo::fill(*src, &constantColor[0]);
+            src->reset(newspec);
+            ImageBufAlgo::fill(*src, constantColor);
             if (verbose) {
                 outstream << "  Constant color image detected. ";
                 outstream << "Creating " << newspec.width << "x"
@@ -1344,8 +1440,9 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
             outstream
                 << "  Alpha==1 image detected. Dropping the alpha channel.\n";
         std::shared_ptr<ImageBuf> newsrc(new ImageBuf(src->spec()));
-        ImageBufAlgo::channels(*newsrc, *src, src->nchannels() - 1, NULL, NULL,
-                               NULL, true);
+        ImageBufAlgo::channels(*newsrc, *src, src->nchannels() - 1,
+                               cspan<int>(), cspan<float>(),
+                               cspan<std::string>(), true);
         std::swap(src, newsrc);  // N.B. the old src will delete
     }
 
@@ -1361,11 +1458,12 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         && pixel_stats.avg[0] == pixel_stats.avg[2]
         && ImageBufAlgo::isMonochrome(*src)) {
         if (verbose)
-            print(
+            OIIO::print(
                 outstream,
                 "  Monochrome image detected. Converting to single channel texture.\n");
         std::shared_ptr<ImageBuf> newsrc(new ImageBuf(src->spec()));
-        ImageBufAlgo::channels(*newsrc, *src, 1, NULL, NULL, NULL, true);
+        ImageBufAlgo::channels(*newsrc, *src, 1, cspan<int>(), cspan<float>(),
+                               cspan<std::string>(), true);
         newsrc->specmod().default_channel_names();
         std::swap(src, newsrc);
     }
@@ -1377,8 +1475,8 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
             outstream << "  Overriding number of channels to " << nchannels
                       << std::endl;
         std::shared_ptr<ImageBuf> newsrc(new ImageBuf(src->spec()));
-        ImageBufAlgo::channels(*newsrc, *src, nchannels, NULL, NULL, NULL,
-                               true);
+        ImageBufAlgo::channels(*newsrc, *src, nchannels, cspan<int>(),
+                               cspan<float>(), cspan<std::string>(), true);
         std::swap(src, newsrc);
     }
 
@@ -1621,14 +1719,14 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         }
 
         ColorConfig colorconfig(colorconfigname);
-        if (colorconfig.error()) {
+        if (colorconfig.has_error()) {
             errorfmt("Error Creating ColorConfig: {}", colorconfig.geterror());
             return false;
         }
 
         ColorProcessorHandle processor
             = colorconfig.createColorProcessor(incolorspace, outcolorspace);
-        if (!processor || colorconfig.error()) {
+        if (!processor || colorconfig.has_error()) {
             errorfmt("Error Creating Color Processor: {}",
                      colorconfig.geterror());
             return false;
@@ -1647,9 +1745,8 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         if (isConstantColor) {
             if (constantColor.size() < 3)
                 constantColor.resize(3, constantColor[0]);
-            if (!ImageBufAlgo::colorconvert(
-                    &constantColor[0], static_cast<int>(constantColor.size()),
-                    processor.get(), unpremult)) {
+            if (!ImageBufAlgo::colorconvert(constantColor, processor.get(),
+                                            unpremult)) {
                 errorfmt("Error applying color conversion to constant color.");
                 return false;
             }
@@ -1658,10 +1755,8 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         if (compute_average_color) {
             if (pixel_stats.avg.size() < 3)
                 pixel_stats.avg.resize(3, pixel_stats.avg[0]);
-            if (!ImageBufAlgo::colorconvert(&pixel_stats.avg[0],
-                                            static_cast<int>(
-                                                pixel_stats.avg.size()),
-                                            processor.get(), unpremult)) {
+            if (!ImageBufAlgo::colorconvert(pixel_stats.avg, processor.get(),
+                                            unpremult)) {
                 errorfmt("Error applying color conversion to average color.");
                 return false;
             }
@@ -1728,15 +1823,16 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
     } else if (!do_resize) {
         // Need format conversion, but no resize -- just copy the pixels
         if (verbose)
-            print(outstream, "  Copying for format conversion from {} to {}\n",
-                  src->spec().format, dstspec.format);
+            OIIO::print(outstream,
+                        "  Copying for format conversion from {} to {}\n",
+                        src->spec().format, dstspec.format);
         toplevel.reset(new ImageBuf(dstspec));
         toplevel->copy_pixels(*src);
     } else {
         // Resize
         if (verbose)
-            print(outstream, "  Resizing image to {} x {}\n", dstspec.width,
-                  dstspec.height);
+            OIIO::print(outstream, "  Resizing image to {} x {}\n",
+                        dstspec.width, dstspec.height);
         string_view resize_filter(filtername);
         if (Strutil::istarts_with(resize_filter, "unsharp-"))
             resize_filter = "lanczos3";
@@ -1754,7 +1850,8 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
                 errorfmt("Could not make filter \"{}\"", resize_filter);
                 return false;
             }
-            ImageBufAlgo::resize(*toplevel, *src, filter);
+            ImageBufAlgo::resize(*toplevel, *src,
+                                 { make_pv("filterptr", filter) });
             Filter2D::destroy(filter);
         }
     }
@@ -1806,6 +1903,9 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
     if (configspec.get_int_attribute("maketx:highlightcomp", 0))
         addlHashData << "highlightcomp=1 ";
 
+    if (configspec.get_int_attribute("maketx:keepaspect", 0))
+        addlHashData << "keepaspect=1 ";
+
     const int sha1_blocksize = 256;
     std::string hash_digest
         = configspec.get_int_attribute("maketx:hash", 1)
@@ -1876,9 +1976,14 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         dstspec.attribute("ImageDescription", desc);
     }
 
+    float aspect_ratio = float(srcspec.full_width) / float(srcspec.full_height);
+
     if (configspec.get_float_attribute("fovcot") == 0.0f) {
-        configspec.attribute("fovcot", float(srcspec.full_width)
-                                           / float(srcspec.full_height));
+        configspec.attribute("fovcot", aspect_ratio);
+    }
+
+    if (configspec.get_int_attribute("maketx:keepaspect")) {
+        configspec.attribute("PixelAspectRatio", aspect_ratio);
     }
 
     maketx_merge_spec(dstspec, configspec);
@@ -1913,22 +2018,24 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
     if (verbose || configspec.get_int_attribute("maketx:runstats")
         || configspec.get_int_attribute("maketx:stats")) {
         double all = alltime();
-        print(outstream, "maketx run time (seconds): {:5.2f}\n", all);
-        print(outstream, "  file read:       {:5.2f}\n", stat_readtime);
-        print(outstream, "  file write:      {:5.2f}\n", stat_writetime);
-        print(outstream, "  initial resize:  {:5.2f}\n", stat_resizetime);
-        print(outstream, "  hash:            {:5.2f}\n", stat_hashtime);
-        print(outstream, "  pixelstats:      {:5.2f}\n", stat_pixelstatstime);
-        print(outstream, "  mip computation: {:5.2f}\n", stat_miptime);
-        print(outstream, "  color convert:   {:5.2f}\n", stat_colorconverttime);
-        print(
+        OIIO::print(outstream, "maketx run time (seconds): {:5.2f}\n", all);
+        OIIO::print(outstream, "  file read:       {:5.2f}\n", stat_readtime);
+        OIIO::print(outstream, "  file write:      {:5.2f}\n", stat_writetime);
+        OIIO::print(outstream, "  initial resize:  {:5.2f}\n", stat_resizetime);
+        OIIO::print(outstream, "  hash:            {:5.2f}\n", stat_hashtime);
+        OIIO::print(outstream, "  pixelstats:      {:5.2f}\n",
+                    stat_pixelstatstime);
+        OIIO::print(outstream, "  mip computation: {:5.2f}\n", stat_miptime);
+        OIIO::print(outstream, "  color convert:   {:5.2f}\n",
+                    stat_colorconverttime);
+        OIIO::print(
             outstream,
             "  unaccounted:     {:5.2f}  ({:5.2f} {:5.2f} {:5.2f} {:5.2f} {:5.2f})\n",
             all - stat_readtime - stat_writetime - stat_resizetime
                 - stat_hashtime - stat_miptime,
             misc_time_1, misc_time_2, misc_time_3, misc_time_4, misc_time_5);
-        print(outstream, "maketx peak memory used: {}\n",
-              Strutil::memformat(peak_mem));
+        OIIO::print(outstream, "maketx peak memory used: {}\n",
+                    Strutil::memformat(peak_mem));
     }
 
 #undef STATUS
